"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.patchExperimentsByExperimentId = exports.deleteExperimentsByExperimentId = exports.getExperimentsByExperimentId = exports.postExperiments = exports.getExperiments = void 0;
const create_1 = require("../database/methods/create");
const delete_1 = require("../database/methods/delete");
const find_1 = require("../database/methods/find");
const format_1 = require("../database/methods/format");
const save_1 = require("../database/methods/save");
const write_1 = require("../database/methods/write");
const errors_1 = require("../types/errors");
const experimentStatus_1 = require("../util/experimentStatus");
const logger_1 = require("../util/logger");
/**
 * This function implements the functionality for handling GET requests on
 * /experiments endpoint.
 * @param _user The user submitting the request.
 */
const getExperiments = async (_user) => {
    logger_1.logger.log('info', `Handling GET request on endpoint /experiments`);
    const experiments = await (0, find_1.findAllExperimentModels)();
    const result = {
        status: 200,
        body: experiments.map((experiment) => (0, format_1.formatExperimentModel)(experiment)),
    };
    logger_1.logger.log('info', `Successfully handled GET request on endpoint /experiments`);
    return result;
};
exports.getExperiments = getExperiments;
/**
 * This function implements the functionality for handling POST requests on
 * /experiments endpoint.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const postExperiments = async (body, _user) => {
    logger_1.logger.log('info', `Handling POST request on endpoint /experiments`);
    const experimentModel = (0, create_1.createExperimentModel)(body);
    const requestedStatus = body.status;
    await (0, save_1.saveExperimentModel)(experimentModel);
    if (requestedStatus === 'booked')
        await (0, experimentStatus_1.bookExperiment)(experimentModel);
    if (requestedStatus === 'running')
        await (0, experimentStatus_1.runExperiment)(experimentModel);
    if (requestedStatus === 'finished')
        await (0, experimentStatus_1.finishExperiment)(experimentModel);
    await (0, save_1.saveExperimentModel)(experimentModel); // NOTE: truly needed?
    const result = {
        status: 201,
        body: (0, format_1.formatExperimentModel)(experimentModel),
    };
    logger_1.logger.log('info', `Successfully handled POST request on endpoint /experiments`);
    return result;
};
exports.postExperiments = postExperiments;
/**
 * This function implements the functionality for handling GET requests on
 * /experiments/{experiment_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 */
const getExperimentsByExperimentId = async (parameters, _user) => {
    logger_1.logger.log('info', `Handling GET request on endpoint /experiments/${parameters.experiment_id}`);
    const experimentModel = await (0, find_1.findExperimentModelById)(parameters.experiment_id);
    if (!experimentModel) {
        throw new errors_1.MissingEntityError(`Could not find experiment model ${parameters.experiment_id}`, 404);
    }
    const result = {
        status: 200,
        body: (0, format_1.formatExperimentModel)(experimentModel),
    };
    logger_1.logger.log('info', `Successfully handled GET request on endpoint /experiments/${parameters.experiment_id}`);
    return result;
};
exports.getExperimentsByExperimentId = getExperimentsByExperimentId;
/**
 * This function implements the functionality for handling DELETE requests on
 * /experiments/{experiment_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 */
const deleteExperimentsByExperimentId = async (parameters, _user) => {
    logger_1.logger.log('info', `Handling DELETE request on endpoint /experiments/${parameters.experiment_id}`);
    const experimentModel = await (0, find_1.findExperimentModelById)(parameters.experiment_id);
    if (!experimentModel)
        throw new errors_1.MissingEntityError(`Could not find experiment model ${parameters.experiment_id}`, 404);
    if (experimentModel?.status !== 'finished')
        await (0, experimentStatus_1.finishExperiment)(experimentModel);
    const result = await (0, delete_1.deleteExperimentModelById)(parameters.experiment_id);
    if (!result.affected) {
        throw new errors_1.MissingEntityError(`Could not find experiment model ${parameters.experiment_id}`, 404);
    }
    if (result.affected > 1) {
        throw new errors_1.InconsistentDatabaseError(`More than one experiment model with id ${parameters.experiment_id} deleted`, 500);
    }
    logger_1.logger.log('info', `Successfully handled DELETE request on endpoint /experiments/${parameters.experiment_id}`);
    return {
        status: 204,
    };
};
exports.deleteExperimentsByExperimentId = deleteExperimentsByExperimentId;
/**
 * This function implements the functionality for handling PATCH requests on
 * /experiment/{experiment_id} endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const patchExperimentsByExperimentId = async (parameters, body, _user) => {
    logger_1.logger.log('info', `Handling PATCH request on endpoint /experiments/${parameters.experiment_id}`);
    const experimentModel = await (0, find_1.findExperimentModelById)(parameters.experiment_id);
    if (!experimentModel) {
        throw new errors_1.MissingEntityError(`Could not find experiment model ${parameters.experiment_id}`, 404);
    }
    if (body)
        (0, write_1.writeExperimentModel)(experimentModel, body);
    if (experimentModel.status === 'booked')
        await (0, experimentStatus_1.bookExperiment)(experimentModel);
    if (experimentModel.status === 'running')
        await (0, experimentStatus_1.runExperiment)(experimentModel);
    if (experimentModel.status === 'finished')
        await (0, experimentStatus_1.finishExperiment)(experimentModel);
    await (0, save_1.saveExperimentModel)(experimentModel);
    const result = {
        status: 200,
        body: (0, format_1.formatExperimentModel)(experimentModel),
    };
    logger_1.logger.log('info', `Successfully handled PATCH request on endpoint /experiments/${parameters.experiment_id}`);
    return result;
};
exports.patchExperimentsByExperimentId = patchExperimentsByExperimentId;
//# sourceMappingURL=experiments.js.map