"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPeerconnection = exports.unlockDevices = exports.deleteBooking = exports.updateDevice = exports.startCloudDeviceInstance = exports.deletePeerconnection = exports.getBooking = exports.lockBooking = exports.bookExperiment = exports.createPeerconnection = exports.instantiateDevice = exports.getDevice = exports.apiClient = void 0;
const config_1 = require("../config");
const errors_1 = require("../types/errors");
const api_client_1 = require("@cross-lab-project/api-client");
const node_fetch_1 = __importDefault(require("node-fetch"));
exports.apiClient = new api_client_1.APIClient(config_1.config.BASE_URL);
/**
 * TODO: add more information on request/response
 * This function handles errors thrown by an internal api request.
 * @param error The error thrown by the internal api request.
 * @throws {InternalRequestError} Thrown if the error matches an error that can be thrown by an internal api request.
 */
function handleInternalRequestError(error) {
    if (error instanceof api_client_1.FetchError)
        throw new errors_1.InternalRequestError(`An error occurred while trying to fetch the request`, error, 500);
    else if (error instanceof api_client_1.ValidationError)
        throw new errors_1.InternalRequestError(`An error occurred while trying to validate the request/response`, error, 500);
    else if (error instanceof api_client_1.InvalidUrlError)
        throw new errors_1.InternalRequestError(`The provided url is malformed`, error, 400);
    else if (error instanceof api_client_1.UnsuccessfulRequestError)
        throw new errors_1.InternalRequestError(`The request was unsuccessful`, error, error.response.status);
}
/**
 * This function wraps the getDevice() function of the api-client and throws server-specific errors.
 * @param args The arguments of the getDevice() function.
 * @throws {InternalRequestError} Thrown if an error occurs during the request.
 * @returns The requested device.
 */
async function getDevice(...args) {
    try {
        return await exports.apiClient.getDevice(args[0], args[1]);
    }
    catch (error) {
        handleInternalRequestError(error);
        throw error;
    }
}
exports.getDevice = getDevice;
/**
 * This function wraps the instantiateDevice() function of the api-client and throws server-specific errors.
 * @param args The arguments of the instantiateDevice() function.
 * @throws {InternalRequestError} Thrown if an error occurs during the request.
 * @returns The created instance of the device and its device token.
 */
async function instantiateDevice(...args) {
    try {
        return await exports.apiClient.instantiateDevice(args[0], args[1]);
    }
    catch (error) {
        handleInternalRequestError(error);
        throw error;
    }
}
exports.instantiateDevice = instantiateDevice;
/**
 * This function wraps the createPeerconnection() function of the api-client and throws server-specific errors.
 * @param args The arguments of the createPeerconnection() function.
 * @throws {InternalRequestError} Thrown if an error occurs during the request.
 * @returns The created peerconnection.
 */
async function createPeerconnection(...args) {
    try {
        return await exports.apiClient.createPeerconnection(args[0], args[1]);
    }
    catch (error) {
        handleInternalRequestError(error);
        throw error;
    }
}
exports.createPeerconnection = createPeerconnection;
/**
 * This function wraps the bookExperiment() function of the api-client and throws server-specific errors.
 * @param args The arguments of the bookExperiment() function.
 * @throws {InternalRequestError} Thrown if an error occurs during the request.
 * @returns The id of the created booking.
 */
async function bookExperiment(...args) {
    try {
        return await exports.apiClient.bookExperiment(args[0], args[1]);
    }
    catch (error) {
        handleInternalRequestError(error);
        throw error;
    }
}
exports.bookExperiment = bookExperiment;
/**
 * This function wraps the lockBooking() function of the api-client and throws server-specific errors.
 * @param args The arguments of the lockBooking() function.
 * @throws {InternalRequestError} Thrown if an error occurs during the request.
 * @returns The locked booking.
 */
async function lockBooking(...args) {
    try {
        return await exports.apiClient.lockBooking(args[0]);
    }
    catch (error) {
        handleInternalRequestError(error);
        throw error;
    }
}
exports.lockBooking = lockBooking;
/**
 * This function wraps the getBooking() function of the api-client and throws server-specific errors.
 * @param args The arguments of the getBooking() function.
 * @throws {InternalRequestError} Thrown if an error occurs during the request.
 * @returns The requested booking.
 */
async function getBooking(...args) {
    try {
        return await exports.apiClient.getBooking(args[0]);
    }
    catch (error) {
        handleInternalRequestError(error);
        throw error;
    }
}
exports.getBooking = getBooking;
/**
 * This function wraps the deletePeerconnection() function of the api-client and throws server-specific errors.
 * @param args The arguments of the deletePeerconnection() function.
 * @throws {InternalRequestError} Thrown if an error occurs during the request.
 */
async function deletePeerconnection(...args) {
    try {
        return await exports.apiClient.deletePeerconnection(args[0]);
    }
    catch (error) {
        handleInternalRequestError(error);
        throw error;
    }
}
exports.deletePeerconnection = deletePeerconnection;
/**
 * This function attempts to start the instance of a cloud instantiable device.
 * @param device The cloud instantiable device for which to start the instance.
 * @param deviceUrl The url the instance should connect to as a device.
 * @param token The token used for authentication by the instance.
 * @param experimentUrl The url of the experiment the instance should take part in.
 */
async function startCloudDeviceInstance(device, deviceUrl, token, experimentUrl) {
    try {
        if (!device.instantiateUrl)
            throw new errors_1.MissingPropertyError('Resolved instantiable cloud device does not have an instantiate url', 500); // NOTE: error code?
        await (0, node_fetch_1.default)(device.instantiateUrl +
            new URLSearchParams([
                ['device_url', deviceUrl],
                ['token', token],
                ['experiment_url', experimentUrl],
            ]), {
            method: 'POST',
            headers: [
                ['Content-Type', 'application/json'],
                ['Authorization', exports.apiClient.accessToken],
            ],
        });
    }
    catch (error) {
        if (error instanceof Error) {
            throw new api_client_1.FetchError(error.message);
        }
        else if (typeof error === 'string') {
            throw new api_client_1.FetchError(error);
        }
        else {
            throw new api_client_1.FetchError('Something went wrong while trying to fetch the request');
        }
    }
}
exports.startCloudDeviceInstance = startCloudDeviceInstance;
/**
 * This function wraps the updateDevice() function of the api-client and throws server specific errors.
 * @param args The arguments of the updateDevice() function.
 * @throws {InternalRequestError} Thrown if an error occurs during the request.
 * @returns The patched device.
 */
async function updateDevice(...args) {
    try {
        return await exports.apiClient.updateDevice(args[0], args[1], args[2]);
    }
    catch (error) {
        handleInternalRequestError(error);
        throw error;
    }
}
exports.updateDevice = updateDevice;
/**
 * This function wraps the deleteBooking() function of the api-client and throws server specific errors.
 * @param args The arguments of the deleteBooking() function.
 * @throws {InternalRequestError} Thrown if an error occurs during the request.
 */
async function deleteBooking(...args) {
    try {
        return await exports.apiClient.deleteBooking(args[0]);
    }
    catch (error) {
        handleInternalRequestError(error);
        throw error;
    }
}
exports.deleteBooking = deleteBooking;
/**
 * This function wraps the unlockDevices() function of the api-client and throws server specific errors.
 * @param args The arguments of the unlockDevices() function.
 * @throws {InternalRequestError} Thrown if an error occurs during the request.
 */
async function unlockDevices(...args) {
    try {
        return await exports.apiClient.unlockBooking(args[0]);
    }
    catch (error) {
        handleInternalRequestError(error);
        throw error;
    }
}
exports.unlockDevices = unlockDevices;
/**
 * This function wraps the getPeerconnection() function of the api-client and throws server specific errors.
 * @param args The arguments of the getPeerconnection() function.
 * @throws {InternalRequestError} Thrown if an error occurs during the request.
 */
async function getPeerconnection(...args) {
    try {
        return await exports.apiClient.getPeerconnection(args[0]);
    }
    catch (error) {
        handleInternalRequestError(error);
        throw error;
    }
}
exports.getPeerconnection = getPeerconnection;
//# sourceMappingURL=api.js.map