"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.callbackHandling = exports.deviceChangedCallbacks = exports.peerconnectionStatusChangedCallbacks = exports.peerconnectionClosedCallbacks = exports.callbackUrl = void 0;
const config_1 = require("../config");
const find_1 = require("../database/methods/find");
const save_1 = require("../database/methods/save");
const errors_1 = require("../types/errors");
const api_1 = require("./api");
const api_client_1 = require("@cross-lab-project/api-client");
exports.callbackUrl = config_1.config.BASE_URL + (config_1.config.BASE_URL.endsWith('/') ? '' : '/') + `callbacks/experiment`;
exports.peerconnectionClosedCallbacks = [];
exports.peerconnectionStatusChangedCallbacks = [];
exports.deviceChangedCallbacks = [];
/**
 * This function adds the endpoint for incoming callbacks registered by the experiment service.
 * @param app The express application the callback endpoint should be added to.
 */
function callbackHandling(app) {
    // TODO: adapt callback handling after codegen update
    app.post('/callbacks/experiment', async (req, res, next) => {
        try {
            const callback = req.body;
            console.log(callback);
            if (typeof callback !== 'object')
                throw new errors_1.MalformedBodyError('Body of callback is not an object', 400);
            const callbackType = getCallbackType(callback);
            switch (callbackType) {
                case 'event':
                    return res.status(await handleEventCallback(callback)).send();
                default:
                    throw new errors_1.InvalidValueError(`Callbacks of type "${req.body.callbackType}" are not supported`, 400);
            }
        }
        catch (error) {
            return next(error);
        }
    });
}
exports.callbackHandling = callbackHandling;
/**
 * This function attempts to get the type of an incoming callback.
 * @param callback The incoming callback of which to get the type.
 * @throws {MalformedBodyError} Thrown if the callback is malformed.
 * @throws {MissingPropertyError} Thrown if the callback is missing a property.
 * @returns The type of the incoming callback.
 */
function getCallbackType(callback) {
    if (!callback.callbackType) {
        throw new errors_1.MissingPropertyError('Callbacks require property "callbackType"', 400);
    }
    if (typeof callback.callbackType !== 'string') {
        throw new errors_1.MalformedBodyError('Property "callbackType" needs to be of type string', 400);
    }
    return callback.callbackType;
}
/**
 * This function handles an incoming event callback.
 * @param callback The incoming event callback to be handled.
 * @throws {MissingPropertyError} Thrown if the callback is missing a property.
 * @throws {InvalidValueError} Thrown if the type of the event callback is unknown.
 * @returns The status code of the callback response.
 */
async function handleEventCallback(callback) {
    if (!callback.eventType) {
        throw new errors_1.MissingPropertyError('Callbacks of type "event" require property "eventType"', 400);
    }
    if (typeof callback.eventType !== 'string') {
        throw new errors_1.MissingPropertyError('Property "eventType" needs to be of type "string"', 400);
    }
    switch (callback.eventType) {
        case 'peerconnection-status-changed':
            return await handlePeerconnectionStatusChangedEventCallback(callback);
        case 'peerconnection-closed':
            return handlePeerconnectionClosedEventCallback(callback);
        case 'device-changed':
            return handleDeviceChangedEventCallback(callback);
        default:
            throw new errors_1.InvalidValueError(`Event-callbacks of type "${callback.eventType}" are not supported`, 400);
    }
}
/**
 * This function handles an incoming "peerconnection-closed" event callback.
 * @param callback The incoming "peerconnection-closed" callback to be handled.
 * @throws {MalformedBodyError} Thrown if the callback is malformed.
 * @throws {MissingPropertyError} Thrown if the callback is missing a property.
 * @returns The status code for the response to the incoming callback.
 */
function handlePeerconnectionClosedEventCallback(callback) {
    if (!callback.peerconnection) {
        throw new errors_1.MissingPropertyError('Event-callbacks of type "peerconnection-closed" require property "peerconnection"', 400);
    }
    const peerconnection = callback.peerconnection;
    if (!api_client_1.DeviceServiceTypes.isPeerconnection(peerconnection)) {
        throw new errors_1.MalformedBodyError('Property "peerconnection" is malformed', 400);
    }
    if (!peerconnection.url) {
        throw new errors_1.MissingPropertyError('Property "peerconnection" is missing property "url"', 400);
    }
    if (!exports.peerconnectionClosedCallbacks.includes(peerconnection.url)) {
        return 410;
    }
    // TODO: add peerconnection closed handling
    return 200;
}
/**
 * This function handles an incoming "peerconnection-status-changed" event callback.
 * @param callback The incoming "peerconnection-status-changed" callback to be handled.
 * @throws {MalformedBodyError} Thrown if the callback is malformed.
 * @throws {MissingPropertyError} Thrown if the callback is missing a property.
 * @returns The status code for the response to the incoming callback.
 */
async function handlePeerconnectionStatusChangedEventCallback(callback) {
    if (!callback.peerconnection) {
        throw new errors_1.MissingPropertyError('Event-callbacks of type "peerconnection-status-changed" require property "peerconnection"', 400);
    }
    const peerconnection = callback.peerconnection;
    if (!api_client_1.DeviceServiceTypes.isPeerconnection(peerconnection)) {
        throw new errors_1.MalformedBodyError('Property "peerconnection" is malformed', 400);
    }
    if (!exports.peerconnectionStatusChangedCallbacks.includes(peerconnection.url)) {
        return 410; // TODO: find a solution for this problem (race condition)
    }
    if (!peerconnection.status) {
        throw new errors_1.MissingPropertyError('Property "peerconnection" is missing property "status"');
    }
    // TODO: add peerconnection status changed handling
    const peerconnectionModel = await (0, find_1.findPeerconnectionModelByUrl)(peerconnection.url);
    if (!peerconnectionModel)
        throw new errors_1.MissingEntityError(`No peerconnection model with url ${peerconnection.url} found`, 500); // NOTE: error code
    const experimentModel = peerconnectionModel.experiment;
    if (!experimentModel)
        throw new errors_1.MissingPropertyError(`Peerconnection model is missing property "experiment"`); // NOTE: error code
    switch (peerconnection.status) {
        case 'closed':
            // TODO: handle status closed
            break;
        case 'connected':
            // TODO: handle status connected
            if (!experimentModel.connections)
                throw new errors_1.MissingPropertyError(`Experiment model is missing property "connections"`, 400); // NOTE: error code
            // eslint-disable-next-line no-case-declarations
            let connected = true;
            for (const pc of experimentModel.connections) {
                if ((await (0, api_1.getPeerconnection)(pc.url)).status !== 'connected') {
                    connected = false;
                }
            }
            if (experimentModel.status === 'setup' && connected) {
                experimentModel.status = 'running';
                await (0, save_1.saveExperimentModel)(experimentModel);
            }
            break;
        case 'failed':
            // TODO: handle status failed
            break;
        case 'new':
            // TODO: handle status new
            break;
        case 'connecting':
            // TODO: handle status connecting
            break;
        case 'disconnected':
            // TODO: handle status disconnected
            break;
    }
    return 200;
}
/**
 * This function handles an incoming "device-changed" event callback.
 * @param callback The incoming "device-changed" callback to be handled.
 * @throws {MalformedBodyError} Thrown if the callback is malformed.
 * @throws {MissingPropertyError} Thrown if the callback is missing a property.
 * @returns The status code for the response to the incoming callback.
 */
function handleDeviceChangedEventCallback(callback) {
    if (!callback.device) {
        throw new errors_1.MissingPropertyError('Event-callbacks of type "device-changed" require property "device"', 400);
    }
    const device = callback.device;
    if (!api_client_1.DeviceServiceTypes.isConcreteDevice(callback.device) &&
        !api_client_1.DeviceServiceTypes.isDeviceGroup(callback.device) &&
        !api_client_1.DeviceServiceTypes.isInstantiableBrowserDevice(callback.device) &&
        !api_client_1.DeviceServiceTypes.isInstantiableCloudDevice(callback.device)) {
        throw new errors_1.MalformedBodyError('Property "device" is not a valid device', 400);
    }
    if (!device.url) {
        throw new errors_1.MissingPropertyError('Property "device" is missing property "url"', 400);
    }
    if (!exports.deviceChangedCallbacks.includes(device.url)) {
        return 410;
    }
    console.log(`Device ${device.url} changed!`);
    // TODO: add device changed handling
    return 200;
}
//# sourceMappingURL=callbacks.js.map