"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildConnectionPlan = void 0;
const errors_1 = require("../types/errors");
const logger_1 = require("./logger");
const url_1 = require("./url");
function buildConnectionPlan(experiment) {
    const experimentUrl = (0, url_1.experimentUrlFromId)(experiment.uuid);
    logger_1.logger.log('info', `Building connection plan for experiment ${experimentUrl}`);
    console.log('building connection plan for experiment', experiment.uuid);
    if (!experiment.serviceConfigurations ||
        experiment.serviceConfigurations.length === 0)
        throw new errors_1.MissingPropertyError('Experiment must have a configuration to be run', 400);
    if (!experiment.devices || experiment.devices.length === 0)
        throw new errors_1.MissingPropertyError('Experiment must have a device to be run', 400);
    const pairwiseServiceConfigurations = toPairwiseServiceConfig(experiment);
    const deviceMappedServiceConfigs = mapRoleConfigToDevices(pairwiseServiceConfigurations, experiment);
    const sortedDeviceMappedServiceConfigs = deviceMappedServiceConfigs.map((dmsc) => sortServiceParticipantsByDeviceId(dmsc));
    const peerconnections = {};
    for (const serviceConfig of sortedDeviceMappedServiceConfigs) {
        // HOTFIX: for local services: Don't connect local services to each other
        // TODO: create a new connection type 'local' as opposed to 'webrtc' and handle it correctly
        if (serviceConfig.devices[0].url === serviceConfig.devices[1].url) {
            continue;
        }
        const lookupKey = `${serviceConfig.devices[0].url}::${serviceConfig.devices[1].url}`;
        if (!(lookupKey in peerconnections)) {
            peerconnections[lookupKey] = {
                type: 'webrtc',
                devices: [
                    {
                        url: (0, url_1.getUrlOrInstanceUrl)(serviceConfig.devices[0]),
                    },
                    {
                        url: (0, url_1.getUrlOrInstanceUrl)(serviceConfig.devices[1]),
                    },
                ],
            };
        }
        const peerconnection = peerconnections[lookupKey];
        updateServiceConfig(peerconnection.devices[0], serviceConfig, serviceConfig.participants[0], serviceConfig.participants[1]);
        updateServiceConfig(peerconnection.devices[1], serviceConfig, serviceConfig.participants[1], serviceConfig.participants[0]);
    }
    const peerconnectionsArray = Object.values(peerconnections);
    console.log(JSON.stringify(peerconnectionsArray));
    console.log('connection plan', JSON.stringify(peerconnectionsArray, null, 4));
    return peerconnectionsArray;
}
exports.buildConnectionPlan = buildConnectionPlan;
function sortServiceParticipantsByDeviceId(serviceConfig) {
    const swap = serviceConfig.devices[0] > serviceConfig.devices[1];
    return {
        ...serviceConfig,
        participants: swap
            ? [serviceConfig.participants[1], serviceConfig.participants[0]]
            : [serviceConfig.participants[0], serviceConfig.participants[1]],
        devices: swap
            ? [serviceConfig.devices[1], serviceConfig.devices[0]]
            : [serviceConfig.devices[0], serviceConfig.devices[1]],
    };
}
function updateServiceConfig(device, serviceConfig, participant, remoteParticipant) {
    device.config = device.config ?? {};
    device.config.services = device.config.services ?? [];
    device.config?.services?.push({
        ...serviceConfig.configuration,
        ...participant.config,
        serviceId: participant.serviceId,
        serviceType: serviceConfig.serviceType,
        remoteServiceId: remoteParticipant.serviceId,
    });
}
function mapRoleConfigToDevices(pairwiseServiceConfigurations, experiment) {
    if (!experiment.devices || experiment.devices.length === 0) {
        throw new errors_1.MissingPropertyError('Experiment must have a device to be run', 400);
    }
    const deviceMappedServiceConfigs = [];
    for (const serviceConfig of pairwiseServiceConfigurations) {
        if (!serviceConfig.participants || serviceConfig.participants.length !== 2) {
            throw new errors_1.MissingPropertyError('pairwiseServiceConfigurations must have exactly two participants', 400);
        }
        const devicesA = experiment.devices
            .filter((d) => d.role === serviceConfig.participants[0].role)
            .map((d) => d);
        const devicesB = experiment.devices
            .filter((d) => d.role === serviceConfig.participants[1].role)
            .map((d) => d);
        for (const deviceA of devicesA) {
            for (const deviceB of devicesB) {
                if (deviceA !== deviceB) {
                    deviceMappedServiceConfigs.push({
                        ...serviceConfig,
                        devices: [deviceA, deviceB],
                    });
                }
                else {
                    // TODO: Handle same device
                }
            }
        }
    }
    return deviceMappedServiceConfigs;
}
function toPairwiseServiceConfig(experiment) {
    if (!experiment.serviceConfigurations) {
        throw new errors_1.MissingPropertyError('Experiment must have a configuration to be run', 400);
    }
    const pairwiseServiceConfigurations = [];
    for (const serviceConfig of experiment.serviceConfigurations) {
        const participants = serviceConfig.participants;
        if (participants) {
            for (let i = 0; i < participants.length; i++) {
                for (let j = i + 1; j < participants.length; j++) {
                    const participantA = participants[i];
                    const participantB = participants[j];
                    // TODO: Check for Service Direction
                    pairwiseServiceConfigurations.push({
                        ...serviceConfig,
                        participants: [participantA, participantB],
                    });
                }
            }
        }
    }
    console.log('connection plan', JSON.stringify(pairwiseServiceConfigurations, null, 4));
    return pairwiseServiceConfigurations;
}
//# sourceMappingURL=connectionPlan.js.map